@@ -15,9 +15,12 @@ class FileToColumnEntries:
     file2fragtolunit = dict()
     file2diss = dict()
     file2enzyme = dict()
-    file2fraction = dict()
-    file2label = dict()
     file2source = dict()
+    # TODO not sure why this is needed
+    file2label = dict()
+    # TODO the following lines will be very difficult with labels. Usually a combination of file&label defines the factor
+    #  I saw that you try to keep lists or combined strings here, but maybe hashing a tuple would be easier here.
+    file2fraction = dict()
     file2combined_factors = dict()
     file2technical_rep = dict()
 
@@ -39,6 +42,10 @@ class OpenMS:
                           'TMT129N': 6, 'TMT129C': 7, 'TMT130N': 8, 'TMT130C': 9, 'TMT131': 10}
         self.tmt6plex = {'TMT126': 1, 'TMT127': 2, 'TMT128': 3,
                          'TMT129': 4, 'TMT130': 5, 'TMT131': 6}
+        # TODO What about iTRAQ?
+
+        # TODO How does this work? In OpenMS there are no such modifications. You can have different "isotope mods"
+        #  for light, medium and heavy. E.g. Label:13C(2)15N(2) (K) as light or Dimethyl:2H(2)13C (K) as light
         self.silac3 = {'silac light': 1, 'silac medium': 2, 'silac heavy': 3}
         self.silac2 = {'silac light': 1, 'silac heavy': 2}
 
@@ -55,7 +62,7 @@ class OpenMS:
 
             accession = re.search("AC=(.+?)(;|$)", m).group(1)
             ptm = self._unimod_database.get_by_accession(accession)
-            if ptm != None:
+            if ptm is not None:
                 name = ptm.get_name()
 
             # workaround for missing PP in some sdrf TODO: fix in sdrf spec?
@@ -65,6 +72,7 @@ class OpenMS:
                 pp = re.search("PP=(.+?)(;|$)", m).group(
                     1)  # one of [Anywhere, Protein N-term, Protein C-term, Any N-term, Any C-term
 
+            ta = ""
             if re.search("TA=(.+?)(;|$)", m) is None:  # TODO: missing in sdrf.
                 warning_message = "Warning no TA= specified. Setting to N-term or C-term if possible."
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
@@ -124,13 +132,12 @@ class OpenMS:
                     c.startswith('comment[modification parameters')]  # columns with modification parameters
 
         if not split_by_columns:
-            # get factor columns (except constant ones)
-            factor_cols = [c for ind, c in enumerate(sdrf) if
-                           c.startswith('factor value[') and len(sdrf[c].unique()) > 1]
 
-            # get characteristics columns (except constant ones)
+            factor_cols = [c for ind, c in enumerate(sdrf) if
+                           c.startswith('factor value[') and len(sdrf[c].unique()) >= 1]
+
             characteristics_cols = [c for ind, c in enumerate(sdrf) if
-                                    c.startswith('characteristics[') and len(sdrf[c].unique()) > 1]
+                                    c.startswith('characteristics[') and len(sdrf[c].unique()) >= 1]
             # and remove characteristics columns already present as factor
             characteristics_cols = self.removeRedundantCharacteristics(characteristics_cols, sdrf, factor_cols)
             print('Factor columns: ' + str(factor_cols))
@@ -143,7 +150,7 @@ class OpenMS:
 
         f2c = FileToColumnEntries()
         for row_index, row in sdrf.iterrows():
-            ## extract mods
+            # extract mods
             all_mods = list(row[mod_cols])
             # print(all_mods)
             var_mods = [m for m in all_mods if
@@ -205,8 +212,13 @@ class OpenMS:
                 f2c.file2fragtolunit[raw] = "ppm"
 
             if 'comment[dissociation method]' in row:
-                diss_method = re.search("NT=(.+?)(;|$)", row['comment[dissociation method]']).group(1)
-                f2c.file2diss[raw] = diss_method.upper()
+                if re.search("NT=(.+?)(;|$)", row['comment[dissociation method]']) is not None:
+                    diss_method = re.search("NT=(.+?)(;|$)", row['comment[dissociation method]']).group(1)
+                    f2c.file2diss[raw] = diss_method.upper()
+                else:
+                    warning_message = "No dissociation method provided. Assuming HCD."
+                    self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
+                    f2c.file2diss[raw] = 'HCD'
             else:
                 warning_message = "No dissociation method provided. Assuming HCD."
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
@@ -243,36 +255,33 @@ class OpenMS:
             else:
                 f2c.file2fraction[raw] = "1"
 
-            try:
+            ## TODO try to avoid try catch here. Can't you just check the number of captured groups?
+            if re.search("NT=(.+?)(;|$)", row['comment[label]']) is not None:
                 label = re.search("NT=(.+?)(;|$)", row['comment[label]']).group(1)
-            except Exception as e:
+                f2c.file2label[raw] = [label]
+            else:
                 if 'TMT' in row['comment[label]']:
-                    labels = sdrf[sdrf['comment[data file]'] == raw]['comment[label]'].tolist()
-                    label = ';'.join(labels)
+                    label = sdrf[sdrf['comment[data file]'] == raw]['comment[label]'].tolist()
                 elif 'SILAC' in row['comment[label]']:
-                    labels = sdrf[sdrf['comment[data file]'] == raw]['comment[label]'].tolist()
-                    label = ';'.join(labels)
+                    label = sdrf[sdrf['comment[data file]'] == raw]['comment[label]'].tolist()
                 elif 'label free sample' in row['comment[label]']:
-                    label = 'label free sample'
+                    label = ['label free sample']
                 else:
-                    label = ''  # TODO For else
-            f2c.file2label[raw] = label
+                    label = ['']  # TODO For else
+                f2c.file2label[raw] = label
 
             if not split_by_columns:
                 # extract factors (or characteristics if factors are missing), and generate one condition for
                 # every combination of factor values present in the data
                 combined_factors = self.combine_factors_to_conditions(characteristics_cols, factor_cols, row)
             else:
-                # take only only entries of splitting columns to generate the conditions
+                # take only entries of splitting columns to generate the conditions
                 combined_factors = "|".join(list(row[split_by_columns]))
 
             # add condition from factors as extra column to sdrf so we can easily filter in pandas
             sdrf.at[row_index, "_conditions_from_factors"] = combined_factors
 
-            if raw in f2c.file2combined_factors.keys():
-                f2c.file2combined_factors[raw] = f2c.file2combined_factors[raw] + ';' + combined_factors
-            else:
-                f2c.file2combined_factors[raw] = combined_factors
+            f2c.file2combined_factors[raw + row['comment[label]']] = combined_factors
 
             # print("Combined factors: " + str(combined_factors))
 
@@ -327,7 +336,7 @@ class OpenMS:
             if combined_factors == "":
                 warning_message = "No factors specified. Adding dummy factor used as condition."
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
-                combined_factors = "NONE"
+                combined_factors = None
             else:
                 warning_message = "No factors specified. Adding non-redundant characteristics as factor. Will be used " \
                                   "as condition. "
@@ -348,7 +357,7 @@ class OpenMS:
     def reportWarnings(self, sdrf_file):
         if len(self.warnings) != 0:
             for k, v in self.warnings.items():
-                print('WARNING: "' + k + '" occured ' + str(v) + ' times.')
+                print('WARNING: "' + k + '" occurred ' + str(v) + ' times.')
         print("SUCCESS (WARNINGS=" + str(len(self.warnings)) + "): " + sdrf_file)
 
     def writeTwoTableExperimentalDesign(self, output_filename, sdrf, file2technical_rep, source_name_list,
@@ -380,9 +389,9 @@ class OpenMS:
                     Fraction_group[raw] = fraction_group
             else:
                 Fraction_group[raw] = fraction_group
-            try:
+            if re.search(sample_identifier_re, source_name) is not None:
                 sample = re.search(sample_identifier_re, source_name).group(1)
-            except Exception as e:
+            else:
                 warning_message = "No sample identifier"
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
 
@@ -394,10 +403,10 @@ class OpenMS:
                     sample = sample_id
                     sample_id += 1
 
-            label = file2label[raw].split(';')
+            label = file2label[raw]
             if "label free sample" in label:
                 label = "1"
-            elif "TMT" in file2label[raw]:
+            elif "TMT" in ','.join(file2label[raw]):
                 if len(label) > 11 or 'TMT134N' in label or 'TMT133C' in label or 'TMT133N' \
                         in label or 'TMT132C' in label or 'TMT132N' in label:
                     choice = self.tmt16plex
@@ -409,7 +418,7 @@ class OpenMS:
                     choice = self.tmt6plex
                 label = str(choice[label[label_index[raw]]])
                 label_index[raw] = label_index[raw] + 1
-            elif 'SILAC' in file2label[raw]:
+            elif 'SILAC' in ','.join(file2label[raw]):
                 if len(label) == 3:
                     label = str(self.silac3[label[label_index[raw]].lower()])
                 else:
@@ -421,17 +430,17 @@ class OpenMS:
             else:
                 out = raw
 
-            f.write(Fraction_group[raw] + "\t" + file2fraction[raw] + "\t" + out + "\t" + label + "\t" + str(sample) + "\n")
+            f.write(
+                Fraction_group[raw] + "\t" + file2fraction[raw] + "\t" + out + "\t" + label + "\t" + str(sample) + "\n")
 
         # sample table
         f.write("\n")
-        if 'tmt' in file2label[sdrf["comment[data file]"].tolist()[0]].lower():
+        if 'tmt' in ','.join(map(lambda x: x.lower(), file2label[sdrf["comment[data file]"].tolist()[0]])):
             openms_sample_header = ["Sample", "MSstats_Condition", "MSstats_BioReplicate", "MSstats_Mixture"]
         else:
             openms_sample_header = ["Sample", "MSstats_Condition", "MSstats_BioReplicate"]
         f.write("\t".join(openms_sample_header) + "\n")
         sample_row_written = list()
-        combined_fac_index = dict(zip(sdrf["comment[data file]"], [0] * len(sdrf["comment[data file]"])))
         mixture_identifier = 1
         mixture_raw_tag = {}
         mixture_sample_tag = {}
@@ -440,7 +449,7 @@ class OpenMS:
         for _0, row in sdrf.iterrows():
             raw = row["comment[data file]"]
             source_name = row["source name"]
-            try:
+            if re.search(sample_identifier_re, source_name) is not None:
                 sample = re.search(sample_identifier_re, source_name).group(1)
 
                 # MSstats BioReplicate column needs to be different for samples from different conditions.
@@ -448,7 +457,7 @@ class OpenMS:
                 MSstatsBioReplicate = sample
                 if sample not in BioReplicate:
                     BioReplicate.append(sample)
-            except Exception as e:
+            else:
                 warning_message = "No sample identifier"
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
 
@@ -458,14 +467,11 @@ class OpenMS:
                 if sample not in BioReplicate:
                     BioReplicate.append(sample)
                 MSstatsBioReplicate = str(BioReplicate.index(sample) + 1)
-
-            if 'NONE' in file2combined_factors[raw]:
+            if file2combined_factors[raw + row['comment[label]']] is None:
                 # no factor defined use sample as condition
-                condition = sample
+                condition = source_name
             else:
-                condition = file2combined_factors[raw].split(';')[combined_fac_index[raw]]
-                combined_fac_index[raw] = combined_fac_index[raw] + 1
-
+                condition = file2combined_factors[raw + row['comment[label]']]
             if len(openms_sample_header) == 4:
                 if raw not in mixture_raw_tag.keys():
                     if sample not in mixture_sample_tag.keys():
@@ -496,7 +502,7 @@ class OpenMS:
                                         file2fraction):
         f = open(output_filename, "w+")
         raw_ext_regex = re.compile(r"\.raw$", re.IGNORECASE)
-        if 'tmt' in file2label[sdrf["comment[data file]"].tolist()[0]].lower():
+        if 'tmt' in map(lambda x: x.lower(), file2label[sdrf["comment[data file]"].tolist()[0]]):
             if legacy:
                 open_ms_experimental_design_header = ["Fraction_Group", "Fraction", "Spectra_Filepath",
                                                       "Label", "Sample", "MSstats_Condition",
@@ -516,7 +522,6 @@ class OpenMS:
                                                       "MSstats_BioReplicate"]
 
         f.write("\t".join(open_ms_experimental_design_header) + "\n")
-        combined_fac_index = dict(zip(sdrf["comment[data file]"], [0] * len(sdrf["comment[data file]"])))
         label_index = dict(zip(sdrf["comment[data file]"], [0] * len(sdrf["comment[data file]"])))
         sample_identifier_re = re.compile(r'sample (\d+)$', re.IGNORECASE)
         Fraction_group = {}
@@ -545,7 +550,7 @@ class OpenMS:
             else:
                 Fraction_group[raw] = fraction_group
 
-            try:
+            if re.search(sample_identifier_re, source_name) is not None:
                 sample = re.search(sample_identifier_re, source_name).group(1)
 
                 # MSstats BioReplicate column needs to be different for samples from different conditions.
@@ -553,12 +558,13 @@ class OpenMS:
                 MSstatsBioReplicate = sample
                 if sample not in BioReplicate:
                     BioReplicate.append(sample)
-            except Exception as e:
+            else:
                 warning_message = "No sample number identifier"
                 self.warnings[warning_message] = self.warnings.get(warning_message, 0) + 1
 
                 # Solve non-sample id expression models
                 if source_name in sample_id_map.keys():
+                    # TODO why do you sometimes build the dicts based on filename and sometimes based on source??
                     sample = sample_id_map[source_name]
                 else:
                     sample_id_map[source_name] = sample_id
@@ -568,17 +574,18 @@ class OpenMS:
                     BioReplicate.append(sample)
                 MSstatsBioReplicate = str(BioReplicate.index(sample) + 1)
 
-            if 'NONE' in file2combined_factors[raw]:
-                # no factor defined use sample as condition
-                condition = sample
+            if file2combined_factors[raw + row['comment[label]']] is None:
+                # no factor defined -> use sample as condition
+                condition = source_name
             else:
-                condition = file2combined_factors[raw].split(';')[combined_fac_index[raw]]
-                combined_fac_index[raw] = combined_fac_index[raw] + 1
-            label = file2label[raw].split(';')
+                condition = file2combined_factors[raw + row['comment[label]']]
+
+            # convert sdrf's label to openms's label
+            label = file2label[raw]
             if "label free sample" in label:
                 label = "1"
 
-            elif "TMT" in file2label[raw]:
+            elif "TMT" in ','.join(file2label[raw]):
                 if len(label) > 11 or 'TMT134N' in label or 'TMT133C' in label or 'TMT133N' \
                         in label or 'TMT132C' in label or 'TMT132N' in label:
                     choice = self.tmt16plex
@@ -589,8 +596,11 @@ class OpenMS:
                 else:
                     choice = self.tmt6plex
                 label = str(choice[label[label_index[raw]]])
+                # TODO if at all, this only works if the labels for the same file are consecutive in the SDRF and always
+                #  in the same order as specified in the initial labels dictionary. Very Dangerous!
+                #  This can be avoided the dicts are built based on file&label as key.
                 label_index[raw] = label_index[raw] + 1
-            elif 'SILAC' in file2label[raw]:
+            elif 'SILAC' in ','.join(file2label[raw]):
                 if len(label) == 3:
                     label = str(self.silac3[label[label_index[raw]].lower()])
                 else:
@@ -642,14 +652,18 @@ class OpenMS:
                                           "FragmentMassToleranceUnit", "DissociationMethod", "Enzyme"]
         f.write("\t".join(open_ms_search_settings_header) + "\n")
         raws = list()
+        TMT_mod = {'tmt6plex': ['TMT6plex (K)', 'TMT6plex (N-term)'],
+                   'tmt10plex': ['TMT6plex (K)', 'TMT6plex (N-term)'],
+                   'tmt11plex': ['TMT6plex (K)', 'TMT6plex (N-term)'],
+                   'tmt16plex': ['TMTpro (K)', 'TMTpro (N-term)']}
         for _0, row in sdrf.iterrows():
             URI = row["comment[file uri]"]
             raw = row["comment[data file]"]
             if raw in raws:
                 continue
             raws.append(raw)
-            labels = f2c.file2label[raw].split(';')
-            if 'TMT' in f2c.file2label[raw]:
+            labels = f2c.file2label[raw]
+            if 'TMT' in ','.join(labels):
                 if len(labels) > 11 or 'TMT134N' in labels or 'TMT133C' in labels or 'TMT133N' \
                         in labels or 'TMT132C' in labels or 'TMT132N' in labels:
                     label = 'tmt16plex'
@@ -659,13 +673,17 @@ class OpenMS:
                     label = 'tmt10plex'
                 else:
                     label = 'tmt6plex'
-
-                # add TMT modification as fixed modification
+                # add default TMT modification when sdrf with label not contains TMT modification
                 if 'TMT' not in f2c.file2mods[raw][0] and 'TMT' not in f2c.file2mods[raw][1]:
-                    tmt_fix_mod = ''
-            elif "label free sample" in f2c.file2label[raw]:
+                    tmt_fix_mod = TMT_mod[label]
+                    if f2c.file2mods[raw][0]:
+                        f2c.file2mods[raw] = (','.join(f2c.file2mods[raw][0].split(',').extend(tmt_fix_mod)),
+                                              f2c.file2mods[raw][1])
+                    else:
+                        f2c.file2mods[raw] = (','.join(tmt_fix_mod), f2c.file2mods[raw][1])
+            elif "label free sample" in labels:
                 label = "label free sample"
-            elif "silac" in f2c.file2label[raw]:
+            elif "silac" in labels:
                 label = "SILAC"
             else:
                 pass  # TODO For else
